from dolfin import *

# Mesh and basis functions (function space) (change mesh for 1D/2D)
#mesh = UnitInterval(20)
mesh = UnitSquare(20, 20)
V = FunctionSpace(mesh, "CG", 1)

# Coefficient for boundary condition
class Kappa(Expression):
    def eval(self, values, x):
        if(x[0] < 0.5):
            values[0] = 0.0
        else:
            values[0] = 1.0e6

# Define FEM formulation
u = TrialFunction(V) # Linear combination of basis functions phi_j
v = TestFunction(V) # Basis function phi_i
f = Expression("0.0")
k = 0.1
epsilon = 0.5
kappa = Kappa(V) # Parameter for choosing boundary condition type
u0 = Function(V)
u1 = Function(V)
# Define FEM equation for timestep with backward Euler
a = (u*v + k*epsilon*inner(grad(u), grad(v)))*dx + kappa*u*v*ds
L = (u0*v + k*f*v)*dx

# Represent FEM equation
problem = VariationalProblem(a, L)

# Set initial condition
for v in vertices(mesh):
    x = v.point()[0]
    y = v.point()[1]
    i = v.index()
    if(x > 0.25 and x < 0.75):
        u1.vector()[i] = 1.0
    else:
        u1.vector()[i] = 0.0

file = File("heat.pvd")
file << u1

# Timestepping
T = 10.0
t = 0.0
while(t < T):
    u0.assign(u1)
    # Build linear system and solve for coefficients uh in linear combination
    u1 = problem.solve()
    
    t += k

    # Save solution in ParaView/VTK format
    file << u1

plot(u1, interactive=True) # Comment out to remove plot window
